#include <conio.h>
#include "music.h"
#include "misc.h"
#include "resource.h"

#define SampleChannels 8

int musInitialized = 0, musPlaying = 0;
musMODULE musModule;
int ModChannels;

#ifdef __USE_MIDAS__

#include "midas\midas.h"
#include "midas\mconfig.h"

void musInitMusic(void) {
  midasSetDefaults();
}

void musInitCard( int musiccard ) {
  midasSDNumber = musiccard;
  midasInit();
  musInitialized = 1;
}

void musLoadModule(char* name) {
  gmpLoadS3M( name, 1, NULL, &musModule );
}

void musStartMusic() {
  midasPlayModule(musModule, 0);
  musPlaying = 1;
}

void musStopMusic() {
  if( musPlaying ) {
    midasStopModule(musModule);
    gmpFreeModule(musModule);
  }
}

void musCloseMusic() {
  if( musInitialized ) {
    midasClose();
  }
}

void musLoadConfig( char* filename ) {
  midasLoadConfig( filename );
}

void musSaveConfig( char* filename ) {
  midasSaveConfig( filename );
}

int musChooseCard() {
#ifdef __OS2__
  return 1;
#else
  cputs( "\r\nChoose sound card (ESC exits):\n\r" );
  vidPutChar( '0', 0x9 );
  cputs( ". Autodetect (enviroment variable have to be set)\n\r" );
  for( int i=0; i<NUMSDEVICES; i++ ) {
    vidPutChar( '0'+i+1, 0x9 );
    cprintf( ". Midas 0.50 %s\n\r", midasSoundDevices[i]->name );
  }
  int c;
  do {
    c = getch();
  } while( c<'0' && c>'0'+NUMSDEVICES && c != 27 );
  if( c == 27 ) error("User abort");
  return midasSDNumber = c-'0'-1;
#endif
}

#endif

#ifdef __USE_MIKMOD__

#include <io.h>
#include <fcntl.h>
#include "mikmod\mikmod.h"

static void tickhandler(void)
{
        MP_HandleTick();    /* play 1 tick of the module */
        MD_SetBPM(mp_bpm);
}

void musInitMusic(void) {
  md_mixfreq      = 44100;                     /* standard mixing freq */
  md_dmabufsize   =16384;                     /* standard dma buf size */
  md_mode         = DMODE_16BITS|DMODE_STEREO; /* standard mixing mode */
/*
  ML_RegisterLoader(&load_m15);
  ML_RegisterLoader(&load_mod);
  ML_RegisterLoader(&load_mtm);
*/
  ML_RegisterLoader(&load_s3m);
/*
  ML_RegisterLoader(&load_xm);
  ML_RegisterLoader(&load_stm);
  ML_RegisterLoader(&load_ult);
  ML_RegisterLoader(&load_uni);
  ML_RegisterLoader(&load_xm);
*/

  MD_RegisterDriver(&drv_nos); // _must_ be the first registred driver

#ifdef __OS2__
  MD_RegisterDriver(&drv_os2);
#elif defined(__WIN32__)
  MD_RegisterDriver(&drv_w95);
#else
  MD_RegisterDriver(&drv_ss);
  MD_RegisterDriver(&drv_sb);
  MD_RegisterDriver(&drv_gus);
#endif


  MD_RegisterPlayer(tickhandler);
}

#ifdef __OS2__

TID UpdateTid;

void OS2Update()
{
  while(1) {
    MD_Update();
    DosSleep(50);
  }
}

#endif

void musInitCard( int musiccard ) {
  md_device = musiccard;
#ifdef __OS2__
  if(!MD_Init()){
    md_device = 2; // drv_nos
#endif
  if(!MD_Init()){
    char str[256];
    sprintf( str, "\tCannot initialize sound system for device #%d \n\t(%s)",
             musiccard, myerr );
    error(str);
  }
#ifdef __OS2__
  }
  DosCreateThread( &UpdateTid, (PFNTHREAD)&OS2Update, 0, CREATE_READY, 4096 );
#else
  miscAddTimerProc( &MD_Update );
#endif
/*
  printf("Using %s for %d bit %s %s sound at %u Hz\n\n",
                  md_driver->Name,
                  (md_mode&DMODE_16BITS) ? 16:8,
                  (md_mode&DMODE_INTERP) ? "interpolated":"normal",
                  (md_mode&DMODE_STEREO) ? "stereo":"mono",
                  md_mixfreq);
*/
  musInitialized = 1;
}

void musLoadModule(char* name) {
  FILE* fp = resOpenFile(name);
  if( !(musModule=ML_LoadFP(fp)) ) error("Cannot load module");
  resClose(fp);
  MP_Init(musModule);
/*
                printf( "Songname: %s\n"
                                "Modtype : %s\n"
                                "Periods : %s,%s\n",
                                musModule->songname,
                                musModule->modtype,
                                (musModule->flags&UF_XMPERIODS) ? "XM type" : "mod type",
                                (musModule->flags&UF_LINEAR) ? "Linear" : "Log");
*/
  ModChannels = musModule->numchn;
  md_numchn = ModChannels + SampleChannels;
}

void musSetModule( uchar* gm ) {
}

void musStartMusic() {
  MD_PlayStart();
  musSetVolume(63);
  musPlaying = 1;
}

void musStopMusic() {
  if( musPlaying ) {
    MD_PlayStop();
    ML_Free(musModule);
    musPlaying = 0;
  }
}

void musCloseMusic() {
  if( musInitialized ) {
#ifdef __OS2__
    DosKillThread( UpdateTid );
#endif
    MD_Exit();
  }
}

void musLoadConfig( char* filename ) {
  int h = open( filename, O_RDONLY );
  read( h, &md_device, sizeof(md_device) );
  read( h, &md_mixfreq, sizeof(md_mixfreq) );
  read( h, &md_mode, sizeof(md_mode) );
  close(h);
}

void musSaveConfig( char* filename ) {
  int h = open( filename, O_WRONLY );
  write( h, &md_device, sizeof(md_device) );
  write( h, &md_mixfreq, sizeof(md_mixfreq) );
  write( h, &md_mode, sizeof(md_mode) );
  close(h);
}

int musChooseCard() {
#ifdef __OS2__
  return 1; // drv_os2
#else
  puts( "Choose sound card (ESC exits):\n\r" );
  vidPutChar( '0', 0x9 );
  cputs( ". Autodetect (enviroment variable have to be set)\n\r" );
  for( int i=0; i<5; i++ ) {
    *(uchar*)(0xB8000+(i+4)*160+1) = 0x9;
  }
  MD_InfoDriver();
  int c;
  do {
    c = getch();
  } while( (c<'0' || c>'9') && c != 27 );
  if( c == 27 ) error("User abort");
  return md_device = c-'0';
#endif
}

musSAMPLE musLoadSample( char* name ) {
  FILE* fp = resOpenFile(name);
  return MW_LoadWavFP(fp);
}

void musPlaySample( musSAMPLE s, uchar volume, uchar panning, int frequency ) {
static int CurChannel = ModChannels;
  MD_VoiceSetVolume(CurChannel,volume%64);
  MD_VoiceSetPanning(CurChannel,panning);
  MD_VoiceSetFrequency(CurChannel,frequency);
  MD_VoicePlay(CurChannel,s->handle,0,s->length,0,0,s->flags);
  CurChannel++;
  if( CurChannel >= ModChannels+SampleChannels )
    CurChannel = ModChannels;
}

void musFreeSample( musSAMPLE s ) {
  MW_FreeWav(s);
}

void musSetVolume( int vol )
{
  mp_volume = 1 + vol*98.0/63;
}

int musGetVolume()
{
  return mp_volume*63/100;
}

#endif
